if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local tinsert, sformat = table.insert, string.format
local EffectiveScale = UIParent:GetEffectiveScale()
local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE
local Demons = Whipped.Demons
local CooldownDemon = Demons.CooldownDemon

-- Frames
Whipped.Frame_Cooldown = CreateFrame("StatusBar", nil, Whipped.MasterFrame)
local Frame_Cooldown = Whipped.Frame_Cooldown
local Frame_Cooldown_Icon = CreateFrame("Frame", nil, Frame_Cooldown)

function Whipped:Cooldown_SetupOptions()
	local Cooldown = {
		name = 'Cooldown',
		type = 'group',
		args = {
			Thickness = {
				name = 'Thickness',
				order = 1,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Cooldown.Thickness = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Cooldown.Thickness end
			},
			Padding = {
				name = 'Padding',
				order = 3,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Cooldown.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Cooldown.Padding end
			},
			Show = {
				name = 'Show',
				type = 'toggle',
				order = 4,
				set =
					function(info, input)
						self.db.global.Cooldown.Show = input;
						if input == true then
							Frame_Cooldown:Show()
						else
							Frame_Cooldown:Hide()
						end
						Whipped:UpdateAllFrames()
					end,
				get =
					function(info)
						return self.db.global.Cooldown.Show
					end
			}
		}
	}
	self:AddCategory("Cooldown", Cooldown)
end

function Whipped:Cooldown_OnLoad()
	Frame_Cooldown:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	Frame_Cooldown:SetMinMaxValues(0, 25)
	Frame_Cooldown:SetStatusBarColor(.1,.9,.2)

	Frame_Cooldown_Icon:SetWidth(Whipped.db.global.Cooldown.Thickness * EffectiveScale)
	Frame_Cooldown_Icon:SetHeight(Whipped.db.global.Cooldown.Thickness * EffectiveScale)
	Frame_Cooldown_Icon:SetPoint(Whipped.anchors[Whipped.db.global.Anchor], Frame_Cooldown, Whipped:OppositeAnchor(Whipped.db.global.Anchor))
	
	Frame_Cooldown_Icon.texture = Frame_Cooldown_Icon:CreateTexture(nil, "BACKGROUND")
	Frame_Cooldown_Icon.texture:SetTexture("Interface\\Icons\\warlock_summon_doomguard")
	Frame_Cooldown_Icon.texture:SetAllPoints(true)
	
	if not self.db.global.Cooldown.Show then Frame_Cooldown:Hide() end
end

function Whipped:Cooldown_OnUpdate()
	if CooldownDemon.Summoned then -- Are Summoned or on Cooldown
		local t = GetTime()
		if CooldownDemon.TTL > 0 then -- Living CooldownDemon
			CooldownDemon.TTL = CooldownDemon.CD - t - 155
			Frame_Cooldown:SetValue(CooldownDemon.TTL)
		else -- Dead CooldownDemon
			if CooldownDemon.TTL < 0 then -- Statement block only runs once every summon, Setup for cooldown
				CooldownDemon.TTL = 0
				CooldownDemon.Alive = 0
				Frame_Cooldown:SetStatusBarColor(.1,.1,.9)
			end
			local val = (155 - (CooldownDemon.CD - t)) / 6.2 -- (CD(180) - TTL(25)) / TTL(25) = 2.6
			Frame_Cooldown:SetValue(val)
			if val > 25 then -- Statement block only runs once every cast, Check if off cooldown (reset the whole thing)
				CooldownDemon.Dead = 0
				CooldownDemon.GUID = 0
				CooldownDemon.Summoned = false
				Frame_Cooldown:SetStatusBarColor(.1,.9,.2)
			end
		end
	end
end

function Whipped:Cooldown_UpdateFrames()
	local MasterFrame = self.db.global
	local anchorID = MasterFrame.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	local SoulShards = MasterFrame.SoulShards
	local Imps = MasterFrame.Imps
	local Dreadstalkers = MasterFrame.Dreadstalkers
	local Cooldown = MasterFrame.Cooldown
	local anchorFrame, anchorHelp, x, y, temp = Whipped.Frame_Dreadstalkers
	
	temp = Dreadstalkers
	if not Dreadstalkers.Show then
		if not Imps.Show then
			if SoulShards.Show then
				anchorFrame = Whipped.Frame_SoulShard_1
				temp = SoulShards
			else
				anchorFrame = Whipped.MasterFrame
				temp = MasterFrame
			end
		else
			anchorFrame = Whipped.Frame_Imps_1
			temp = Imps
		end
	end
	
	if anchorID == 1 or anchorID == 2 then
		Frame_Cooldown:SetWidth((SoulShards.Width * 5 + SoulShards.Padding * 4) * EffectiveScale)
		Frame_Cooldown:SetHeight(Cooldown.Thickness * EffectiveScale)
		if anchorID == 1 then
			if temp == MasterFrame then
				anchorHelp, x, y = "TOP", MasterFrame.PaddingX * EffectiveScale, -(MasterFrame.PaddingY + Cooldown.Padding) * EffectiveScale
			else
				anchorHelp, x, y = "BOTTOM", 0, -(temp.Padding + Cooldown.Padding) * EffectiveScale
			end
		else
			if temp == MasterFrame then
				anchorHelp, x, y = "TOP", -MasterFrame.PaddingX * EffectiveScale, -(MasterFrame.PaddingY + Cooldown.Padding) * EffectiveScale
			else
				anchorHelp, x, y = "BOTTOM", 0, -(temp.Padding + Cooldown.Padding) * EffectiveScale
			end
		end
	else
		Frame_Cooldown:SetWidth(Cooldown.Thickness * EffectiveScale)
		Frame_Cooldown:SetHeight((SoulShards.Height * 5 + SoulShards.Padding * 4) * EffectiveScale)
		if anchorID == 3 then
			if temp == MasterFrame then
				anchorHelp, x, y = "LEFT", (MasterFrame.PaddingX + Cooldown.Padding) * EffectiveScale, -MasterFrame.PaddingY * EffectiveScale
			else
				anchorHelp, x, y = "RIGHT", (temp.Padding + Cooldown.Padding) * EffectiveScale, 0
			end
		else
			if temp == MasterFrame then
				anchorHelp, x, y = "LEFT", (MasterFrame.PaddingX + Dreadstalker.Padding) * EffectiveScale, -MasterFrame.PaddingY * EffectiveScale
			else
				anchorHelp, x, y = "RIGHT", (temp.Padding + Cooldown.Padding) * EffectiveScale, 0
			end
		end
	end
	Frame_Cooldown_Icon:SetWidth(Cooldown.Thickness * EffectiveScale)
	Frame_Cooldown_Icon:SetHeight(Cooldown.Thickness * EffectiveScale)
		
	Frame_Cooldown:ClearAllPoints()
	Frame_Cooldown_Icon:ClearAllPoints()
	if anchorID == 1 then
		Frame_Cooldown:SetPoint("TOP"..anchor, anchorFrame, anchorHelp..anchor, x, y)
		Frame_Cooldown_Icon:SetPoint(anchor, Frame_Cooldown, Oanchor, MasterFrame.PaddingX * EffectiveScale, 0)
		Frame_Cooldown:SetOrientation("HORIZONTAL")
	elseif anchorID == 2 then
		Frame_Cooldown:SetPoint("TOP"..anchor, anchorFrame, anchorHelp..anchor, x, y)
		Frame_Cooldown_Icon:SetPoint(anchor, Frame_Cooldown, Oanchor, -MasterFrame.PaddingX * EffectiveScale, 0)
		Frame_Cooldown:SetOrientation("HORIZONTAL")
	elseif anchorID == 3 then
		Frame_Cooldown:SetPoint(anchor.."LEFT", anchorFrame, anchor..anchorHelp, x, y)
		Frame_Cooldown_Icon:SetPoint(anchor, Frame_Cooldown, Oanchor, 0, -MasterFrame.PaddingY * EffectiveScale)
		Frame_Cooldown:SetOrientation("VERTICAL")
	elseif anchorID ==4 then
		Frame_Cooldown:SetPoint(anchor.."LEFT", anchorFrame, anchor..anchorHelp, x, y)
		Frame_Cooldown_Icon:SetPoint(anchor, Frame_Cooldown, Oanchor, 0, MasterFrame.PaddingY * EffectiveScale)
		Frame_Cooldown:SetOrientation("VERTICAL")
	end
end

function Whipped:Cooldown_CLE(...)
	local event = select(2, ...)
	if event == "SPELL_CAST_SUCCESS" then
		if CooldownDemon.Alive > 0 then
			Frame_Cooldown:SetStatusBarColor(1,.1,.1)
		end
	elseif event == "SPELL_AURA_REMOVED" then
		if CooldownDemon.TTL > 0 then
			Frame_Cooldown:SetStatusBarColor(.1,.9,.2)
		end
	elseif event == "SPELL_SUMMON" then
		local SpellID = select(12, ...)
		if (SpellID == 60478 or SpellID == 111685) and bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then -- Doomguard(cd), Infernal(cd)
			CooldownDemon.Summoned = true
			CooldownDemon.CD = GetTime() + 180
			CooldownDemon.TTL = CooldownDemon.CD - 155
			CooldownDemon.Alive = 1
			CooldownDemon.DE = 1
			CooldownDemon.Name = select(9, ...)
			CooldownDemon.GUID = select(8, ...)
		end
	elseif event == "UNIT_DIED" then
		if select(8,...) == CooldownDemon.GUID then -- Doomguard(cd), Infernal(cd)
			if CooldownDemon.Alive > 0 then -- Infernal actually has a UNIT_DIED event when expired, WHY DOES EVERY DEMON WORK COMPLETELY DIFFERENTLY TO THE LAST ONE, c'mon blizz :(
				CooldownDemon.Alive = 0
				CooldownDemon.TTL = 0
				Frame_Cooldown:SetStatusBarColor(.1,.1,.9)
			end
		end
	end
end